"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("./util");
class Decode {
    btou(u) {
        return u.replace(/[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF7][\x80-\xBF]{3}/g, (cccc) => {
            switch (cccc.length) {
                case 4:
                    let cp = ((0x07 & cccc.charCodeAt(0)) << 18)
                        | ((0x3f & cccc.charCodeAt(1)) << 12)
                        | ((0x3f & cccc.charCodeAt(2)) << 6)
                        | (0x3f & cccc.charCodeAt(3)), offset = cp - 0x10000;
                    return (util_1.default.fromCC((offset >>> 10) + 0xD800)
                        + util_1.default.fromCC((offset & 0x3FF) + 0xDC00));
                case 3:
                    return util_1.default.fromCC(((0x0f & cccc.charCodeAt(0)) << 12)
                        | ((0x3f & cccc.charCodeAt(1)) << 6)
                        | (0x3f & cccc.charCodeAt(2)));
                default:
                    return util_1.default.fromCC(((0x1f & cccc.charCodeAt(0)) << 6)
                        | (0x3f & cccc.charCodeAt(1)));
            }
        });
    }
    atob(asc, ch, padchar) {
        let { tab } = util_1.default.getChsTab(ch);
        asc = asc.replace(/\s+/g, '');
        asc += (padchar || '==').slice(2 - (asc.length & 3));
        let u24, bin = '', r1, r2;
        for (let i = 0; i < asc.length;) {
            u24 = tab[asc.charAt(i++)] << 18
                | tab[asc.charAt(i++)] << 12
                | (r1 = tab[asc.charAt(i++)]) << 6
                | (r2 = tab[asc.charAt(i++)]);
            bin += r1 === 64 ? util_1.default.fromCC(u24 >> 16 & 255)
                : r2 === 64 ? util_1.default.fromCC(u24 >> 16 & 255, u24 >> 8 & 255)
                    : util_1.default.fromCC(u24 >> 16 & 255, u24 >> 8 & 255, u24 & 255);
        }
        return bin;
    }
    get(a, ch, padchar) {
        return this.btou(this.atob(a, ch, padchar)).replace(/\x00/ig, '');
    }
    getPaddingIndex(encodeLength) {
        let step = 1;
        let padding;
        switch (encodeLength % 4) {
            case 4:
            case 3:
                step = encodeLength % 2 ? 1 : 3;
                padding = Math.floor(step * encodeLength / 4);
                break;
            case 2:
                step = encodeLength % 2 ? 1 : 2;
                padding = Math.floor(step * encodeLength / 3);
                break;
            case 1:
            default:
                padding = Math.floor(encodeLength / 2);
                break;
        }
        return padding;
    }
    toInter(hex) {
        if (hex) {
            if (hex[0] == '0') {
                hex = hex.substr(1, hex.length);
            }
            return parseInt(hex, 16);
        }
        return 0;
    }
    decode(str, ch, dpad) {
        let code = util_1.default.switch(str.split(''), str.length % 2 ? 0 : 2);
        let paddingIndex = this.getPaddingIndex(code.length - 2);
        let padding = code.splice(paddingIndex, 2);
        let switchIndex = this.toInter(padding.join(''));
        let strSwitch = code.splice(0, code.length - switchIndex);
        code = util_1.default.switch(code, switchIndex % 2 ? 0 : 2).reverse().concat(strSwitch);
        if (switchIndex % 2) {
            code = code.reverse();
        }
        return this.get(code.join(''), ch, dpad);
    }
}
exports.default = new Decode();
